# SHELL_FUNCTIONS.SH
#
# Author: "Garen Erdoisa" <scamper@trisk.com>
# Licence: GNU GPL
# Last update: 8/14/2011
#
# This code is written on behalf of the SpamBouncer project
# and as such is donated by the Author to the SpamBouncer
# project.
#
# Synopsis:
# This is a /bin/sh functions file intended to be loaded
# into the current environemt of "functions/mimeparts.sh"
# which is a SpamBouncer /bin/sh script that uses these functions
# for parsing MIME Version 1.0 encoded email.
# This script is designed specifically for use inside SpamBouncer's
# unique procmail recipes. It is not intended for general use or for
# use with any other program or set of programs, and would require major
# modifications to make it work outside of that environment.

# FUNCTION SPLITPARTS()
# expecting base filename to split into mime parts as the first command line argument.
# expecting a part designation as the 2nd command line argument.
function splitparts(){
	${CAT} $1 |\
	${SED} -ne "$3p" |\
	${SED} -e '2,$!d;$d' >$1_$2

	${CAT} $1 |\
	${SED} -ne "$3p" |\
	${SED} -e '2,$!d;$d' |\
	${FORMAIL} -cX "Content-" |\
	${SED} -e "${SPACES_TABS_TO_SPACES}" |\
	${SED} -e '$s/.*/&\n/' >$1_$2.header

	${CAT} $1 |\
	${SED} -ne "$3p" |\
	${SED} -e '2,$!d;$d' |\
	${FORMAIL} -I "" >$1_$2.body
}

# FUNCTION GREPLOCALPARTS()
# expecting filename in $1 and boundary tag in $2 for input
function greplocalparts()
{
	# Save the boundary tag to a file for a later fgrep
	# using the string in the file. I had to do it this way
	# because fgrep misinterprets strings begining with one or two -'s when
	# they appear on the command line as command line arguments
	# to fgrep, while if they are in a file, fgrep uses them as
	# a pattern to match against. Boundary strings always begining with --
	# in the mime message body.
	${ECHO} "$2" >${1}.boundary

	# Replace any regexp meta characters in the boundary string
	# in the pattern space only with a regexp . wildcard char.
	# This is so sed or sh do not misinterpret the chars as
	# /bin/sh or sed regexp meta characters.
	# The numbers are octal.
	# 042 " shell open/close quoted text
	# 044 $ end of line anchor also shell meta in some cases
	# 052 * sed match zero or more of preceeding character
	# 054 ' forward single quote
	# 057 / sed open/close pattern space.
	# 133 [ regexp char list open also shell meta
	# 134 \ sed escape the next char.
	# 135 ] regexp char list close also shell meta
	# 136 ^ begining of line anchor
	# 140 ` back quote shell execute sub command
	#
	# All of the above are replaced with this in the pattern space.
	# 056 . sed wild card match a single char (used to replace all of these in the pattern space)
	
	LOCSTR=`${ECHO} "${2}" |${TR} "\042\044\052\054\057\133\134\135\136\140" "\056"`

	# Use the modified boundary string with the wildcards to find the
	# starting and ending line numbers of mime parts.
	#
	# We are prepending the line numbers to indicate exactly where in the part
	# the boundary string was found. Trailing whitespaces are allowd.
	# The two greps at the end, confirm that the strings found
	# using this method are valid by comparing them back against the
	# unmodified boundary string.
	LOCALPARTS=`
	 ${CAT} $1 |\
         ${SED} -ne "/^--${LOCSTR}\(--\)\?\([ 	]\{1,\}\)\?$/=;//p" |\
         ${SED} -e 's/$/;/' |\
	 ${TR} -d '\012\015' |\
	 ${SED} -e 's/\([0-9]\{1,\}\)\(;\)\(--\)/\1:\3/g;s/;/\n/g' |\
	 ${GREP} -Faf ${1}.boundary |\
	 ${EGREP} "^[0-9]+:--.+(--)?[ 	]+?$"
	`
}

# FUNCTION GETLINEPAIRS()
# expects a grep of the file with line numbers in front of boundary indicators as quoted input.
# ie:
# 3:--__Boundary
# 9:--__Boundary
# 130:--__Boundary
# 824:--__Boundary--
# It will reformat the above into:
# "3,9 9,130 130,824"
function getlinepairs()
{
# echo "function getlinepairs"
LOCALLINEPAIRS=`
	${ECHO} "$1" |\
	${SED} -ne '
			# Begin surgery on preceeding pattern space from the Grep results:
			#
			# Address line one only of the preceeding pattern space
			# replace the line number from the grep with the line number
			# followed by a comma
			1s/^\([0-9]\{1,\}\):\(.\{1,\}\)$/\1,/p

			# Next we have to duplicate some of the line numbers so
			# we can end up with pairs. The end of one part becomes
			# the defacto begining of the next part in a multi part
			# message.
			#
			# Address line one to end of preceeding pattern space
			# replace first match where P is the match with P;\nP, 
			# drop the 2nd match entirely.
			1,$s/^\([0-9]\{1,\}\):\(.\{1,\}\)$/\1;\n\1,/p
		' |\
	${SED} -e '
			# continued surgery on preceeding pattern space
			# Delete the last line of the preceeding pattern space.
			# we dont need it.
			$d

			# the tr following this strips out end of line characters
			# in the preceeding pattern space, putting everything back
			# just on one line so that we can seperate the lines again
			# where the semi-colons were edited in.
		' |\
	${TR} -d '\012\015' |\
	${SED} -e '
			# continued surgery on preceeding pattern space
			# globally replace all semi-colons with <space>, then strip
			# any trailing space from the pattern space.
			# This makes rows with pairs of line numbers seperated by a comma
			# this format is sutable for use in sed to address just those
			# lines in the main message body.
			# The pairs of line numbers reference the begining and ending
			# lines of each part of the mime boundary we are considering
			# in the message body only. Line 1 being the first line of
			# the message body after the headers.
			s/;/ /g
			s/ $//
		'`
}

# FUNCTION DBECHO()
# Conditional echo.
# first parameter is a number that indicates if we should print the message
# or not. If that number is greater than or equal to the current SBLOGLEVEL
# then we do, otherwise skip it.
function dbecho()
{
	LEVEL="$1"
	if [ ${LEVEL} -le ${SBLOGLEVEL} ] && [ -f ${LOGFILE} ]; then
		shift
		SBLOGPREFIX="`${DATE} '+%b %d %H:%M:%S'` ${HOST} SpamBouncer[${SBPID}]:"
		MESSAGE="${SBLOGPREFIX} $*"
		if [ "${SBLOGFILE}" = "INTERNAL" ]; then
			${ECHO} "${MESSAGE}"
		elif [ -f ${SBLOGFILE} ]; then
			${ECHO} "${MESSAGE}" >>${SBLOGFILE}
		fi
	fi
}
